package com.biasi.partyapp;

import android.app.ProgressDialog;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.transition.ChangeBounds;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;
import com.biasi.partyapp.models.Festa;
import com.biasi.partyapp.mutants.HttpQueue;
import com.biasi.partyapp.util.Util;
import com.biasi.partyapp.util.UtilNet;
import com.nostra13.universalimageloader.core.ImageLoader;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class FestaScreen extends AppCompatActivity {

    private long festaId = -1;
    private String festaUrl;
    private Festa f;
    private Toolbar mToolbar;
    private ImageView image;
    private FloatingActionButton share;
    private TextView data, style, descricao, location_city, website;
    private RelativeLayout site, location;
    private CollapsingToolbarLayout mCollapsingToolbarLayout;
    private Button vo, nao_vo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            getWindow().setSharedElementEnterTransition(new ChangeBounds());
            getWindow().setSharedElementReturnTransition(new ChangeBounds());
        }

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_festa_screen);

        Util.setupImageLoader(this);

        mCollapsingToolbarLayout = (CollapsingToolbarLayout) findViewById(R.id.collapsing_toolbar);
        mCollapsingToolbarLayout.setCollapsedTitleTextColor(getResources().getColor(android.R.color.white));
        mCollapsingToolbarLayout.setExpandedTitleColor(getResources().getColor(android.R.color.white));

        mToolbar = (Toolbar) findViewById(R.id.toolbar_top);
        setSupportActionBar(mToolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        image = (ImageView) findViewById(R.id.festa_image);
        share = (FloatingActionButton) findViewById(R.id.fab);
        data = (TextView) findViewById(R.id.data);
        style = (TextView) findViewById(R.id.style);
        descricao = (TextView) findViewById(R.id.description);
        location = (RelativeLayout) findViewById(R.id.item_location);
        location_city = (TextView) findViewById(R.id.text_location);
        site = (RelativeLayout) findViewById(R.id.item_site);
        website = (TextView) findViewById(R.id.website);
        //vo = (Button) findViewById(R.id.vo);
        //nao_vo = (Button) findViewById(R.id.nao_vo);


        // Instância vinda do próprio app ou de notificação.
        festaUrl = String.valueOf(getIntent().getExtras().getString("festa"));
        Pattern pat = Pattern.compile("[0-9]+");
        Matcher match = pat.matcher(festaUrl);
        if (match.find()) {
            festaId = Long.parseLong(match.group(0));
        } else {
            Log.e(Util.TAG, "Identifier not found");
            finish();
            return;
        }

        f = Festa.findById(Festa.class, festaId);

        if (f == null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(false);
            getPartyAsync();
        } else {
            setData();
        }
    }

    private void setData() {
        Calendar cal = new GregorianCalendar();
        cal.setTimeInMillis(f.getTimeStamp());
        data.setText(String.format(getString(R.string.data), cal.get(Calendar.DAY_OF_MONTH), cal.get(Calendar.MONTH) + 1, cal.get(Calendar.HOUR_OF_DAY), cal.get(Calendar.MINUTE)));

        String[] styles = getResources().getStringArray(R.array.estilos);
        style.setText(styles[f.getStyle()]);

        descricao.setText(f.getDescricao());
        website.setText(f.getSite());
        mToolbar.setTitle(f.getTitulo());
        mCollapsingToolbarLayout.setTitle(f.getTitulo());

        ImageLoader.getInstance().displayImage(f.getBannerUrl(), image);

        location.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(FestaScreen.this, MapScreen.class);
                i.putExtra("festa_id", f.getId());
                startActivity(i);
            }
        });

        String city = Util.getCityName(this, ""+f.getCidade());
        location_city.setText(String.format(getString(R.string.see_google_maps), city));

        site.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(Intent.ACTION_VIEW, Uri.parse(f.getSite()));
                startActivity(i);
            }
        });

        share.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(Intent.ACTION_SEND);
                i.putExtra(Intent.EXTRA_TEXT, String.format("%s, acesse a festa pelo aplicativo Party!", f.getTitulo()));
                i.setType("text/plain");
                startActivity(Intent.createChooser(i, getString(R.string.share_with)));
            }
        });

        /*vo.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Snackbar.make(findViewById(android.R.id.content), "Você vai!!", Snackbar.LENGTH_SHORT).show();
            }
        });

        nao_vo.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Snackbar.make(findViewById(android.R.id.content), "Você não vai :(", Snackbar.LENGTH_SHORT).show();
            }
        });*/
        UtilNet.countRequest(this, f);
    }


    private void getPartyAsync() {
        final ProgressDialog cx = ProgressDialog.show(this, null, getString(R.string.searching), true, false);
        StringRequest request = new StringRequest(Request.Method.POST, String.format(UtilNet.partyUrl,festaId), new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                if (Util.ResponseToDatabase(response) > 0) {
                    f = Festa.findById(Festa.class, festaId);
                    setData();
                } else {
                    Toast.makeText(FestaScreen.this, R.string.request_error, Toast.LENGTH_SHORT).show();
                    FestaScreen.this.finish();
                }
                cx.dismiss();
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                cx.dismiss();
                Log.e(Util.TAG, "StatusCode="+error.networkResponse.statusCode);
                Toast.makeText(FestaScreen.this, R.string.request_error, Toast.LENGTH_SHORT).show();
                FestaScreen.this.finish();
            }
        });
        HttpQueue.getInstance(this).add(request);
    }


    @Override
    public boolean onOptionsItemSelected(MenuItem menuItem) {
        if (menuItem.getItemId() == android.R.id.home) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                finishAfterTransition();
            } else {
                finish();
            }
        }
        return super.onOptionsItemSelected(menuItem);
    }

}
