package com.biasi.partyapp.adapters;

import android.content.Context;
import android.content.SharedPreferences;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.biasi.partyapp.R;
import com.biasi.partyapp.interfaces.RecyclerViewOnClickListener;
import com.biasi.partyapp.models.Festa;
import com.biasi.partyapp.util.Util;
import com.nostra13.universalimageloader.core.ImageLoader;

import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.GregorianCalendar;
import java.util.List;

public class FestaAdapter extends RecyclerView.Adapter<FestaAdapter.MyViewHolder> {

    private List<Festa> mList;
    private LayoutInflater mLayoutInflater;
    private RecyclerViewOnClickListener clickListener;
    private ImageLoader mImageLoader;
    private TextView emptyView;
    private Context mContext;

    public FestaAdapter(Context context, TextView emptyView) {
        this.mContext = context;
        this.emptyView = emptyView;
        this.mImageLoader = ImageLoader.getInstance();
        this.mLayoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        update();
    }

    public void update() {
        mList = Festa.listAll(Festa.class);
        Collections.sort(mList);
        notifyDataSetChanged();

        if (getItemCount() == 0) {
            emptyView.setVisibility(View.VISIBLE);
            SharedPreferences pref = Util.getConfigs(mContext);

            int distance = pref.getInt("gps_range", 5);
            String city_name = Util.getCityName(mContext, pref.getString("city", "5003702"));

            String style_id = pref.getString("style", "0");
            String style = style_id.equals("0") ? "" : " de " + Util.getStyleName(mContext, style_id) + "";

            int mode_id = Util.getViewMode(mContext);
            String mode;
            switch (mode_id) {
                case Util.TODAY:
                    mode = " " + mContext.getString(R.string.msg_today) + " ";
                    break;
                case Util.TOMORROW:
                    mode = " " + mContext.getString(R.string.msg_tomorrow) + " ";
                    break;
                case Util.MONTH:
                    mode = " " + mContext.getString(R.string.msg_month) + " ";
                    break;
                case Util.ALL:
                    mode = " " + mContext.getString(R.string.msg_all) + " ";
                    break;
                default:
                    mode = "(?)";
                    break;
            }

            String cidade = pref.getBoolean("gps_mode", true) ? String.format(mContext.getString(R.string.dist), distance) : "em " + city_name + ".";

            String result = String.format(mContext.getString(R.string.empty_list), style, mode, cidade);

            emptyView.setText(result);
        } else {
            emptyView.setVisibility(View.GONE);
        }
    }

    @Override
    public int getItemCount() {
        return mList.size();
    }

    public void setRecyclerViewOnClickListener(RecyclerViewOnClickListener r) {
        clickListener = r;
    }

    @Override
    public long getItemId(int position) {
        return mList.get(position).getId();
    }

    @Override
    public MyViewHolder onCreateViewHolder(ViewGroup viewGroup, int viewType) {
        View v = mLayoutInflater.inflate(R.layout.item_festa_card, viewGroup, false);
        return new MyViewHolder(v);
    }

    @Override
    public void onBindViewHolder(final MyViewHolder myViewHolder, int position) {
        final Festa f = mList.get(position);
        myViewHolder.titulo.setText(f.getTitulo());
        myViewHolder.descricao.setText(f.getDescricao());

        Calendar data = new GregorianCalendar();
        data.setTimeInMillis(f.getTimeStamp());

        myViewHolder.data1.setText(String.format("%02d/%02d", data.get(Calendar.DAY_OF_MONTH),data.get(Calendar.MONTH) + 1));
        myViewHolder.data2.setText(Util.getStyleName(mContext, f.getStyle() + ""));

        mImageLoader.displayImage(f.getBannerUrl(), myViewHolder.banner);

        myViewHolder.banner.setVerticalFadingEdgeEnabled(true);
        myViewHolder.banner.setFadingEdgeLength(40);
    }


    public class MyViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private ImageView banner;
        private TextView titulo, descricao, data1, data2;

        public MyViewHolder(View itemView) {
            super(itemView);
            banner = (ImageView) itemView.findViewById(R.id.image1);
            titulo = (TextView) itemView.findViewById(R.id.text1);
            descricao = (TextView) itemView.findViewById(R.id.text2);
            data1 = (TextView) itemView.findViewById(R.id.data1);
            data2 = (TextView) itemView.findViewById(R.id.data2);
            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            if (clickListener != null) {
                clickListener.onClick(v, getPosition());
            }
        }
    }
}
