package com.biasi.partyapp.util;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.XmlResourceParser;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.util.Log;

import com.biasi.partyapp.R;
import com.biasi.partyapp.models.Festa;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.xmlpull.v1.XmlPullParser;

import java.io.File;
import java.io.FileInputStream;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Util {

    public static final String TAG = "Party";
    public static final String avatar = "avatar.jpg";
    public static final String cover = "cover.jpg";
    public static final String conf = "whisky";

    public static final int TODAY = 0;
    public static final int TOMORROW = 1;
    public static final int MONTH = 2;
    public static final int ALL = 3;

    private static boolean loadedImageLoader = false;

    public static SharedPreferences getConfigs(Context context) {
        return context.getSharedPreferences(conf, Context.MODE_PRIVATE);
    }

    public static boolean isLogged(Context context) {
        return getConfigs(context).getBoolean("isRegistred", false);
    }

    public static Drawable getImage(Context context, String uri) {
        try {
            File f = new File(context.getFilesDir(), uri);
            Bitmap b = BitmapFactory.decodeStream(new FileInputStream(f));
            return new BitmapDrawable(context.getResources(), b);
        } catch (Exception e) {
            return context.getResources().getDrawable(R.drawable.vazio);
        }
    }

    public static String getTitle(Context context) {
        switch (getViewMode(context)) {
            case Util.TODAY:
                return context.getString(R.string.today);
            case Util.TOMORROW:
                return context.getString(R.string.tomorrow);
            case Util.MONTH:
                return context.getString(R.string.month);
            case Util.ALL:
                return context.getString(R.string.all);
            default:
                return "(?)";
        }
    }

    public static String getSubtitle(Context context) {
        SharedPreferences pref = getConfigs(context);
        String style_id = pref.getString("style", "0");
        String style = style_id.equals("0") ? "" : getStyleName(context, style_id) + " ";
        if (isGPSMode(context)) {
            int range = pref.getInt("gps_range", 5);
            return String.format(context.getString(R.string.gps_subtitle), style, range);
        } else {
            String id = getCityId(context);
            return String.format(context.getString(R.string.city_subtitle), style, getCityName(context, id));
        }
    }

    public static void setCityStateId(Context context, long cityId, long stateId) {
        try {
            getConfigs(context).edit().putString("city", String.valueOf(cityId)).apply();
            getConfigs(context).edit().putString("state", String.valueOf(stateId)).apply();
        } catch (Exception e) {
            Log.e("BIASI", "Error on city set");
        }
    }

    public static String getCityId(Context context) {
        return getConfigs(context).getString("city", "5003702");
    }

    public static String getStateId(Context context) {
        return getConfigs(context).getString("state", "50");
    }

    public static String getCityName(Context context, String city_id) {
        String cidade = "?", estado = "?", estado_id = "-1";
        try {
            XmlResourceParser cidades = context.getResources().getXml(R.xml.cidades);
            while (cidades.getEventType()!= XmlPullParser.END_DOCUMENT) {
                if (cidades.getEventType() == XmlPullParser.START_TAG) {
                    if (cidades.getName().equals("cidade") && cidades.getAttributeValue(0).equals(city_id)) {
                        estado_id = cidades.getAttributeValue(1);
                        cidades.next();
                        cidade = cidades.getText();
                        break;
                    }
                }
                cidades.next();
            }
        }
        catch (Exception t) {
            t.printStackTrace();
        }

        try {
            XmlResourceParser estados = context.getResources().getXml(R.xml.estados);
            while (estados.getEventType()!= XmlPullParser.END_DOCUMENT) {
                if (estados.getEventType() == XmlPullParser.START_TAG) {
                    if (estados.getName().equals("estado") && estados.getAttributeValue(0).equals(estado_id)) {
                        estados.next();
                        estado = estados.getText();
                        break;
                    }
                }
                estados.next();
            }
        }
        catch (Exception t) {
            t.printStackTrace();
        }

        return cidade + " - " + estado;
    }

    public static String getStyleName(Context context, String style_id) {
        String[] estilos_names = context.getResources().getStringArray(R.array.estilos);
        String[] estilos_values = context.getResources().getStringArray(R.array.estilos_values);
        for (int i = 0; i < estilos_values.length; i++) {
            if (estilos_values[i].equals(style_id)) {
                return estilos_names[i];
            }
        }
        return "(?)";
    }


    public static int ResponseToDatabase(String response) {
        Pattern pat = Pattern.compile("[-]?[0-9]+.[0-9]+");
        try {
            JSONArray array = new JSONArray(response);
            int count = 0;
            for (int i = 0; i < array.length(); i++) {
                JSONObject obj = array.getJSONObject(i);

                Festa f = Festa.findById(Festa.class, obj.getLong("pk"));

                if (f == null) {
                    // Nova festa!
                    f = new Festa();
                    count++;
                }
                f.setId(obj.getLong("pk"));

                JSONObject fields = obj.getJSONObject("fields");

                f.setTitulo(fields.getString("titulo"));
                f.setDescricao(fields.getString("descricao"));
                f.setBannerUrl(UtilNet.mediaUrl + fields.getString("banner"));
                f.setStyle(fields.getInt("style"));

                DateFormat formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'", Locale.US);
                formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
                Calendar cal = GregorianCalendar.getInstance();
                cal.setTime(formatter.parse(fields.getString("data")));
                f.setTimeStamp(cal.getTimeInMillis());

                f.setCidade(fields.getLong("cidade"));
                f.setSite(fields.getString("site"));

                Matcher match = pat.matcher(fields.getString("location"));
                match.find();
                if (match.find())
                    f.setLocalLongitude(Double.parseDouble(match.group(0)));
                if (match.find())
                    f.setLocalLatitude(Double.parseDouble(match.group(0)));

                f.save();
            }
            return count;
        } catch (JSONException | ParseException e) {
            Log.e(Util.TAG, e.getMessage());
            return -1;
        }
    }

    public static void setCurrentLocation(Context context, Location l) {
        SharedPreferences.Editor editor = getConfigs(context).edit();
        editor.putString("latitude", String.valueOf(l.getLatitude()));
        editor.putString("longitude", String.valueOf(l.getLongitude()));
        Log.i("BIASI", "GPS Locked: (" + l.getLatitude() + "," + l.getLongitude() + ")");
        editor.apply();
    }

    public static int getViewMode(Context context) {
        return getConfigs(context).getInt("viewMode", Util.TODAY);
    }

    public static void setViewMode(Context context, int mode) {
        getConfigs(context).edit().putInt("viewMode", mode).apply();
    }

    public static boolean isGPSMode(Context context) {
        return getConfigs(context).getBoolean("gps_mode", true);
    }

    public static void setGPSMode(Context context, boolean mode) {
        getConfigs(context).edit().putBoolean("gps_mode", mode).apply();
    }

    public static void setupImageLoader(Context context) {
        if (!loadedImageLoader) {
            loadedImageLoader = true;
            DisplayImageOptions displayOptions = new DisplayImageOptions.Builder()
                    .showImageForEmptyUri(R.drawable.loading)
                    .showImageOnLoading(R.drawable.loading)
                    .showImageOnFail(R.drawable.loading)
                    .cacheInMemory(true)
                    .cacheOnDisk(true)
                    .build();

            ImageLoaderConfiguration conf = new ImageLoaderConfiguration.Builder(context)
                    .defaultDisplayImageOptions(displayOptions)
                    .memoryCacheSize(2 * 1024 * 1024)
                    .diskCacheSize(10 * 1024 * 1024)
                    .threadPoolSize(4)
                    .writeDebugLogs()
                    .build();

            ImageLoader.getInstance().init(conf);
        }
    }
}